// -*- C++ -*-
/*!
 * @file  Houghlines.cpp
 * @brief Draw Houghlines
 * $Date$
 *
 * $Id$
 */

#include "Houghlines.h"
//#include "Camera.h"


// Module specification
// <rtc-template block="module_spec">
static const char* houghlines_spec[] =
  {
    "implementation_id", "Houghlines",
    "type_name",         "Houghlines",
    "description",       "Draw Houghlines",
    "version",           "0.1",
    "vendor",            "Osaka Univ.",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_param", "0",
    "conf.default.int_width","320",
    "conf.default.int_height","240",
    "conf.default.int_type", "0",

    ""
  };
// </rtc-template>

Houghlines::Houghlines(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InputImageIn("InputImage", m_InputImage),
    m_OutputImageOut("OutputImage", m_OutputImage),

    // </rtc-template>
    m_src(0), m_osrc(0), m_dst(0), m_color_dst(0),
    dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InputImage", m_InputImageIn);
  
  // Set OutPort buffer
  registerOutPort("OutputImage", m_OutputImageOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

	m_src = NULL;
  m_osrc = NULL;
  m_dst = NULL;
  m_color_dst = NULL;

}

Houghlines::~Houghlines()
{
}


RTC::ReturnCode_t Houghlines::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_param", m_int_param, "0");
  bindParameter("int_type", m_int_type, "0");
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");

  //printf("initialize\n");
  //cvNamedWindow("Source", 0);
  //cvNamedWindow("Processed", 0);
  // </rtc-template>
  return RTC::RTC_OK;
}




RTC::ReturnCode_t Houghlines::onFinalize()
{
  //cvDestroyWindow("Source");
  //cvDestroyWindow("Processed");

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Houghlines::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Houghlines::onActivated(RTC::UniqueId ec_id)
{

  m_src       = cvCreateImage(cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  m_osrc      = cvCreateImage(cvGetSize(m_src),IPL_DEPTH_8U, 3);
  m_dst       = cvCreateImage(cvGetSize(m_src), 8, 1 );
  m_color_dst = cvCreateImage(cvGetSize(m_src), 8, 3 );
  
  return RTC::RTC_OK;
}




RTC::ReturnCode_t Houghlines::onDeactivated(RTC::UniqueId ec_id)
{
  cvReleaseImage(&m_src);
  cvReleaseImage(&m_osrc);
  cvReleaseImage(&m_dst);
  cvReleaseImage(&m_color_dst);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t Houghlines::onExecute(RTC::UniqueId ec_id)
{

  CvSeq* lines = 0;
  unsigned int i=0;

  //if(!src)
  //  return RTC::RTC_OK;
  //printf("1\n");

  /*inport part begin*/
  m_InputImageIn.read();

  if(m_InputImage.data.length() == m_int_width * m_int_height * 3) {
    memcpy(m_osrc->imageData,
           (void *)&(m_InputImage.data[0]),
           m_int_width * m_int_height * 3);
    /*inport part end*/
    
    /*main process begin*/
    cvCvtColor(m_osrc, m_src, CV_BGR2GRAY);
    cvCanny(m_src, m_dst, 50, 200, 3);
    cvCvtColor(m_dst, m_color_dst, CV_GRAY2BGR);
    //copy src to color_dst
    if (m_int_type == 0) {
      CvMemStorage* storage = cvCreateMemStorage(0);
      lines = cvHoughLines2(m_dst, storage, CV_HOUGH_STANDARD,
                            1, CV_PI/180, 100, 0, 0 );
      
      for ( i = 0; i < MIN(lines->total,100); i++ ) {
        float* line = (float*)cvGetSeqElem(lines,i);
        float rho = line[0];
        float theta = line[1];
        CvPoint pt1, pt2;
        double a = cos(theta), b = sin(theta);
        double x0 = a*rho, y0 = b*rho;
        pt1.x = cvRound(x0 + 1000*(-b));
        pt1.y = cvRound(y0 + 1000*(a));
        pt2.x = cvRound(x0 - 1000*(-b));
        pt2.y = cvRound(y0 - 1000*(a));
        //cvLine( color_dst, pt1, pt2, CV_RGB(255,0,0), 3, CV_AA, 0 );
        cvLine( m_osrc, pt1, pt2, CV_RGB(255,0,0), m_int_param, CV_AA, 0 );
        cvReleaseMemStorage (&storage);
      }
    } else {
      CvMemStorage* storage = cvCreateMemStorage(0);
      lines = cvHoughLines2( m_dst, storage, CV_HOUGH_PROBABILISTIC,
                             1, CV_PI/180, 50, 50, 10 );
      for( int i = 0; i < lines->total; i++ )  {
        CvPoint* line = (CvPoint*)cvGetSeqElem(lines,i);
        cvLine( m_osrc , line[0], line[1], CV_RGB(255,0,0),
                m_int_param, CV_AA, 0 );
        //cvLine( color_dst, line[0], line[1], CV_RGB(255,0,0), 3, CV_AA, 0 );
      }
      cvReleaseMemStorage (&storage);
    }
    /*main process end*/
    
    /*outport part begin*/
    //cvShowImage("Source", src);
    //cvShowImage("Processed", color_dst);
    //m_OutputImage.data.length(color_dst->imageSize);
    m_OutputImage.data.length(m_osrc->imageSize);
    memcpy((void *)&(m_OutputImage.data[0]),
           m_osrc->imageData,
           m_osrc->imageSize);
    
    m_OutputImageOut.write();
    /*outport part end*/
  }
  cvWaitKey(100);
  
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t Houghlines::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Houghlines::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void HoughlinesInit(RTC::Manager* manager)
  {
    RTC::Properties profile(houghlines_spec);
    manager->registerFactory(profile,
                             RTC::Create<Houghlines>,
                             RTC::Delete<Houghlines>);
  }
  
};


