// -*- C++ -*-
/*!
 * @file  CameraComponent.cpp
 * @brief Camera Component
 * $Date$
 *
 * $Id$
 */

#include "CameraComponent.h"
//#include "Camera.h"


// Module specification
// <rtc-template block="module_spec">
static const char* cameracomponent_spec[] =
  {
    "implementation_id", "CameraComponent",
    "type_name",         "CameraComponent",
    "description",       "Camera Component",
    "version",           "0.1",
    "vendor",            "Osaka Univ.",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width","320",
    "conf.default.int_height","240",
    "conf.default.int_Flip","1",

    ""
  };
// </rtc-template>

CameraComponent::CameraComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_ImageDataOut("ImageData", m_ImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("ImageData", m_ImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

	src = NULL;
	//buf2 = NULL;
	//buf3 = NULL;

	capture = 0;

}

CameraComponent::~CameraComponent()
{
}


RTC::ReturnCode_t CameraComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable

  
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_Flip", m_int_Flip, "1");

  //cam.Initialize();
  
  /*
    capture = cvCaptureFromCAM( 0);
    if( !capture )
    {
      //fprintf(stderr,"Could not initialize capturing...\n");
        printf("Could not initialize capturing...\n");
    }
    cvSetCaptureProperty(capture,CV_CAP_PROP_FRAME_WIDTH,m_int_width);
    cvSetCaptureProperty(capture,CV_CAP_PROP_FRAME_HEIGHT,m_int_height);
  */

   printf( "Do activate : \n"
            "\for graving image from cam\n");

 
   //cvNamedWindow("Capture", 0);//CV_WINDOW_AUTOSIZE);
  


  // </rtc-template>
  return RTC::RTC_OK;
}




RTC::ReturnCode_t CameraComponent::onFinalize()
{
  //cam.Close();
  cvReleaseCapture(&capture);
  //cvDestroyWindow("Capture");

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t CameraComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t CameraComponent::onActivated(RTC::UniqueId ec_id)
{
  capture = cvCaptureFromCAM( -1 );

    // JȂ̏
  if(capture == NULL){
  	printf("Cannot Open Camera Device!!\n");
		return RTC::RTC_ERROR;  // G[I
  }


  if( !capture )
    {
      //fprintf(stderr,"Could not initialize capturing...\n");
      printf("Could not initialize capturing...\n");
    }
  
  cvSetCaptureProperty(capture,CV_CAP_PROP_FRAME_WIDTH,m_int_width);
  cvSetCaptureProperty(capture,CV_CAP_PROP_FRAME_HEIGHT,m_int_height);
  printf("%d %d \n",m_int_width, m_int_height);

  printf( "If you'd like to change resolution, change width and height \n\n");

	printf("͉摜𔽓]ꍇ́A[int_Flip]̒lύXĉ.\n");
	printf("0:]Ȃ 1:] 2:] 3:]\n\n");

  return RTC::RTC_OK;
}



RTC::ReturnCode_t CameraComponent::onDeactivated(RTC::UniqueId ec_id)
{
  cvReleaseCapture(&capture);
  return RTC::RTC_OK;
}



RTC::ReturnCode_t CameraComponent::onExecute(RTC::UniqueId ec_id)
{
  IplImage *tmp_img = NULL;


  src=cvQueryFrame(capture);

	if( m_int_Flip == 1 ){
		tmp_img = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);
		cvCopy(src, tmp_img, NULL);
		// ]
		cvFlip (tmp_img, src, 0);
	}
	else if( m_int_Flip == 2 ){
		tmp_img = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);
		cvCopy(src, tmp_img, NULL);
		// ]
		cvFlip (tmp_img, src, 1);
	}
	else if( m_int_Flip == 3 ){
		tmp_img = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);
		cvCopy(src, tmp_img, NULL);
		// ]
		cvFlip (tmp_img, src, -1);
	}
  cvReleaseImage(&tmp_img);

  m_ImageData.data.length(src->imageSize);
  memcpy((void *)&(m_ImageData.data[0]),src->imageData,src->imageSize);


  /*
  for(int i=0;i<src->imageSize;i++){
    m_ImageData.data[i]=src->imageData[i];
  }
  */

  //m_ImageData.data[0]=src->imageData[0];
 
  m_ImageDataOut.write();
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t CameraComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void CameraComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(cameracomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<CameraComponent>,
                             RTC::Delete<CameraComponent>);
  }
  
};


