// -*- C++ -*-
/*!
 * @file  ContoursComponent.cpp
 * @brief Contours Component
 * $Date$
 *
 * $Id$
 */

#include "ContoursComponent.h"
//#include "Camera.h"

// Module specification
// <rtc-template block="module_spec">
static const char* contourscomponent_spec[] =
  {
    "implementation_id", "ContoursComponent",
    "type_name",         "ContoursComponent",
    "description",       "Contours Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_type1", "1",
    "conf.default.int_type2", "1",
    "conf.default.int_param", "1",

    ""
  };
// </rtc-template>

ContoursComponent::ContoursComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    m_osrc(0), m_dst(0),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

	m_osrc = NULL;
  m_dst = NULL;

}

ContoursComponent::~ContoursComponent()
{
}


RTC::ReturnCode_t ContoursComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_type1", m_int_type1, "1");
  bindParameter("int_type2", m_int_type2, "1");
  bindParameter("int_param", m_int_param, "1");
  
  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t ContoursComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ContoursComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ContoursComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ContoursComponent::onActivated(RTC::UniqueId ec_id)
{
  m_osrc = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 3);
  m_dst = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t ContoursComponent::onDeactivated(RTC::UniqueId ec_id)
{
  cvReleaseImage(&m_osrc);
  cvReleaseImage(&m_dst);
  return RTC::RTC_OK;
}



RTC::ReturnCode_t ContoursComponent::onExecute(RTC::UniqueId ec_id)
{

  int type1=0,type2=0;

  /*inport part*/
  m_InImageDataIn.read();
  if(m_InImageData.data.length() == m_int_width * m_int_height * 3) {
    memcpy(m_osrc->imageData,
           (void *)&(m_InImageData.data[0]),
           m_int_width * m_int_height * 3);

    if(m_int_type1 < 4) {
      switch(m_int_type1)
	{
	case 0:
	  type1=CV_RETR_EXTERNAL;
	  break;
	case 1:
	  type1=CV_RETR_LIST;
	  break;
	case 2:
	  type1=CV_RETR_CCOMP;
	  break;
	case 3:
	  type1=CV_RETR_TREE;
	  break;
	}
    }
    if(m_int_type2 < 5) {
      switch(m_int_type2)
	{
	case 0:
	  type2=CV_CHAIN_CODE;
	  break;
	case 1:
	  type2=CV_CHAIN_APPROX_NONE;
	  break;
	case 2:
	  type2=CV_CHAIN_APPROX_SIMPLE;
	  break;
	case 3:
	  type2=CV_CHAIN_APPROX_TC89_L1;
	  break;
	case 4:
	  type2=CV_LINK_RUNS;
	  break;
	}
    }
    
    cvCvtColor(m_osrc, m_dst, CV_BGR2GRAY);
    {
      int contour_num;
      CvSeq* contours = NULL;
      CvMemStorage* storage = cvCreateMemStorage(0);
      contour_num = cvFindContours(m_dst, storage, &contours,
                                   sizeof(CvContour),
                                   type1, type2, cvPoint(0,0));

      cvDrawContours(m_osrc, contours, CV_RGB(255,0,0), CV_RGB(255,0,0),
                     1, m_int_param, 8);
      cvReleaseMemStorage (&storage);
    }
    
    /*outport part begin*/
    m_OutImageData.data.length(m_int_width * m_int_height * 3);
    memcpy((void *)&(m_OutImageData.data[0]),
           m_osrc->imageData,
           m_int_width * m_int_height * 3);
    m_OutImageDataOut.write();
    /*outport part end*/
  }
  cvWaitKey(100);
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t ContoursComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ContoursComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ContoursComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ContoursComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ContoursComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ContoursComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(contourscomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<ContoursComponent>,
                             RTC::Delete<ContoursComponent>);
  }
  
};


