// -*- C++ -*-
/*!
 * @file  DbackComponent.cpp
 * @brief Daynamic Background Component
 * $Date$
 *
 * $Id$
 */

#include "DbackComponent.h"
//#include "Camera.h"


// Module specification
// <rtc-template block="module_spec">
static const char* dbackcomponent_spec[] =
  {
    "implementation_id", "DbackComponent",
    "type_name",         "DbackComponent",
    "description",       "Daynamic Background Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_st", "3",
    "conf.default.int_th", "30",

    ""
  };
// </rtc-template>

DbackComponent::DbackComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

	src = NULL;
	osrc = NULL;
	dst = NULL;
	cdst = NULL;
	bdst = NULL;
	ddst = NULL;
	odst = NULL;
	mask = NULL;

}

DbackComponent::~DbackComponent()
{
}


RTC::ReturnCode_t DbackComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_st", m_int_st, "3");
  bindParameter("int_th", m_int_th, "30");

  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t DbackComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DbackComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DbackComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t DbackComponent::onActivated(RTC::UniqueId ec_id)
{
  //  int i;

  src = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  osrc = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;

  mask = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  dst= cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  odst= cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 1);
  cdst = cvCreateImage( cvGetSize(src),IPL_DEPTH_8U, 3);//3);//NULL;
  bdst = cvCreateImage( cvGetSize(src),IPL_DEPTH_32F, 3);//3);//NULL;
 
  /*inport part begin*/
  m_InImageDataIn.read();
  if(m_InImageData.data.length()>0){
    memcpy(osrc->imageData,(void *)&(m_InImageData.data[0]),m_int_width*m_int_height*3);//m_InImageData.data.length());
  }  
  /*inport part end*/

  //cvCopy(osrc,cdst);

  return RTC::RTC_OK;
}


RTC::ReturnCode_t DbackComponent::onDeactivated(RTC::UniqueId ec_id)
{


  return RTC::RTC_OK;
}


RTC::ReturnCode_t DbackComponent::onExecute(RTC::UniqueId ec_id)
{
  //int i;

  /*inport part begin*/
  m_InImageDataIn.read();
  if(m_InImageData.data.length()>0){
    memcpy(osrc->imageData,(void *)&(m_InImageData.data[0]),m_int_width*m_int_height*3);//m_InImageData.data.length());
  }
  /*inport part end*/


  for(int i=0;i<m_int_width*m_int_height;i++){
    unsigned char *new_p=(unsigned char *)&osrc->imageData[i*3];
    unsigned char new_b=new_p[0];
    unsigned char new_g=new_p[1];
    unsigned char new_r=new_p[2];

    float *bg_p=(float *)&bdst->imageData[i*3*sizeof(float)];
    int bg_b=(int)bg_p[0];
    int bg_g=(int)bg_p[1];
    int bg_r=(int)bg_p[2];
 
    int diff =abs( (int)new_b -(int)bg_b)+abs((int)new_g-(int)bg_g)+abs((int)new_r-(int)bg_r);

    if(diff>m_int_th){
	mask->imageData[i]=0;
    }else{
	mask->imageData[i]=1;
    }
  }

  ALPHA=(float)m_int_st*0.1;
  if(ALPHA>1.0)ALPHA=1.0;
  else if(ALPHA<0.0)ALPHA=0.0;
  cvRunningAvg(osrc,bdst,ALPHA);

  /*color*/
  memset(cdst->imageData, 0, m_int_width*m_int_height*3);
  cvCopy(cdst,osrc,mask);


  /*outport part begin*/
  m_OutImageData.data.length(osrc->imageSize);
  /*
  for(int i=0;i<cdst->imageSize;i++){
    //for(int i=0;i<color_dst->imageSize;i++){
    m_OutImageData.data[i]=osrc->imageData[i];
  }
  */
  memcpy((void *)&(m_OutImageData.data[0]),osrc->imageData,m_int_width*m_int_height*3);//m_InImageData.data.length());
  m_OutImageDataOut.write();
  /*outport part end*/

  cvWaitKey(100);


  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t DbackComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DbackComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DbackComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DbackComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DbackComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void DbackComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(dbackcomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<DbackComponent>,
                             RTC::Delete<DbackComponent>);
  }
  
};


