// -*- C++ -*-
/*!
 * @file  DilateComponent.cpp
 * @brief Dilate Component
 * $Date$
 *
 * $Id$
 */

#include "DilateComponent.h"
//#include "Camera.h"

// Module specification
// <rtc-template block="module_spec">
static const char* dilatecomponent_spec[] =
  {
    "implementation_id", "DilateComponent",
    "type_name",         "DilateComponent",
    "description",       "Dilate Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_param", "1",

    ""
  };
// </rtc-template>

DilateComponent::DilateComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

	osrc = NULL;
	dst = NULL;

}

DilateComponent::~DilateComponent()
{
}


RTC::ReturnCode_t DilateComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_param", m_int_param, "1");
  
  times=m_int_param;

  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t DilateComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DilateComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DilateComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t DilateComponent::onActivated(RTC::UniqueId ec_id)
{
  osrc = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 3);
  dst = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 3);
  return RTC::RTC_OK;
}



RTC::ReturnCode_t DilateComponent::onDeactivated(RTC::UniqueId ec_id)
{
  cvReleaseImage(&osrc);
  cvReleaseImage(&dst);
  return RTC::RTC_OK;
}



RTC::ReturnCode_t DilateComponent::onExecute(RTC::UniqueId ec_id)
{
  /*inport part*/
  m_InImageDataIn.read();
  if(m_InImageData.data.length()==m_int_width*m_int_height*3){
    memcpy(osrc->imageData,(void *)&(m_InImageData.data[0]),m_int_width*m_int_height*3);

    times=m_int_param;
    if(times<1)
      times=1;
    cvDilate(osrc,dst,NULL,times);

    /*outport part begin*/
    m_OutImageData.data.length(m_int_width*m_int_height*3);
    memcpy((void *)&(m_OutImageData.data[0]),dst->imageData,m_int_width*m_int_height*3);
    m_OutImageDataOut.write();
    /*outport part end*/
  }


  cvWaitKey(100);
  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t DilateComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DilateComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DilateComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DilateComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DilateComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void DilateComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(dilatecomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<DilateComponent>,
                             RTC::Delete<DilateComponent>);
  }
  
};


