// -*- C++ -*-
/*!
 * @file  FlowComponent.cpp
 * @brief Optical Flow Component
 * $Date$
 *
 * $Id$
 */

#include "FlowComponent.h"
//#include "Camera.h"

// Module specification
// <rtc-template block="module_spec">
static const char* flowcomponent_spec[] =
  {
    "implementation_id", "FlowComponent",
    "type_name",         "FlowComponent",
    "description",       "Optical Flow Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_param", "11",

    ""
  };
// </rtc-template>

FlowComponent::FlowComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    m_osrc(0), m_src(0), m_vel_x(0), m_vel_y(0),
    // </rtc-template>
    dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  m_osrc = NULL;
  m_src = NULL;
  m_src_pre = NULL;
  m_vel_x = NULL;
  m_vel_y = NULL;

}

FlowComponent::~FlowComponent()
{
}


RTC::ReturnCode_t FlowComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_param", m_int_param, "11");
  
  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t FlowComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FlowComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FlowComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t FlowComponent::onActivated(RTC::UniqueId ec_id)
{

  m_osrc    = cvCreateImage(cvSize(m_int_width,m_int_height), IPL_DEPTH_8U, 3);
  m_src     = cvCreateImage(cvSize(m_int_width,m_int_height), IPL_DEPTH_8U, 1);
  m_src_pre = cvCreateImage(cvSize(m_int_width,m_int_height), IPL_DEPTH_8U, 1);
  m_vel_x   = cvCreateImage(cvSize(m_int_width,m_int_height), IPL_DEPTH_32F, 1);
  m_vel_y   = cvCreateImage(cvSize(m_int_width,m_int_height), IPL_DEPTH_32F, 1);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t FlowComponent::onDeactivated(RTC::UniqueId ec_id)
{
  cvReleaseImage(&m_osrc);
  cvReleaseImage(&m_src);
  cvReleaseImage(&m_src_pre);

  cvReleaseImage(&m_vel_x);
  cvReleaseImage(&m_vel_y);
  return RTC::RTC_OK;
}



RTC::ReturnCode_t FlowComponent::onExecute(RTC::UniqueId ec_id)
{
  /*inport part*/
  m_InImageDataIn.read();

  if(m_InImageData.data.length() == m_int_width * m_int_height * 3) {
    memcpy(m_osrc->imageData,
           (void *)&(m_InImageData.data[0]),
           m_int_width * m_int_height * 3);

    cvCvtColor(m_osrc, m_src, CV_BGR2GRAY);

    cvCalcOpticalFlowLK(m_src_pre, m_src, cvSize(m_int_param, m_int_param),
                        m_vel_x, m_vel_y);

    // ̏̂ߌ݂̃C[WRs[
    cvCopy(m_src, m_src_pre, NULL);
    
    for(int i(0); i < m_int_height; i+=(m_int_param-1)) {
      for(int j(0); j < m_int_width; j+=(m_int_param-1)) {
        int dx;
        int dy;
	dx = (int)cvGetReal2D(m_vel_x, i, j);
	dy = (int)cvGetReal2D(m_vel_y, i, j);
	cvLine(m_osrc, cvPoint(j,i), cvPoint(j+dx,i+dy),
               CV_RGB(255,255,255), 1, 8, 0);
	cvRectangle(m_osrc, cvPoint(j + dx - 1, i + dy - 1),
                    cvPoint(j+dx+1,i+dy+1),
                    CV_RGB(255,255,255), 1, 8, 0);
      }
    }

    /*outport part begin*/
    m_OutImageData.data.length(m_int_width * m_int_height * 3);
    memcpy((void *)&(m_OutImageData.data[0]), 
           m_osrc->imageData,
           m_int_width * m_int_height * 3);

    m_OutImageDataOut.write();
    /*outport part end*/
  }

  cvWaitKey(1);

  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t FlowComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FlowComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FlowComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FlowComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FlowComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void FlowComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(flowcomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<FlowComponent>,
                             RTC::Delete<FlowComponent>);
  }
  
};


