// -*- C++ -*-
/*!
 * @file  SmoothComponent.cpp
 * @brief Smooth Component
 * $Date$
 *
 * $Id$
 */

#include "SmoothComponent.h"
//#include "Camera.h"



// Module specification
// <rtc-template block="module_spec">
static const char* smoothcomponent_spec[] =
  {
    "implementation_id", "SmoothComponent",
    "type_name",         "SmoothComponent",
    "description",       "Smooth Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_param1", "5",
    "conf.default.int_param2", "0",
    "conf.default.int_param3", "0",
    "conf.default.int_type", "2",

    ""
  };
// </rtc-template>

SmoothComponent::SmoothComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

	osrc = NULL;
	dst = NULL;

}

SmoothComponent::~SmoothComponent()
{
}


RTC::ReturnCode_t SmoothComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_param1", m_int_param1, "5");
  bindParameter("int_param2", m_int_param2, "0");
  bindParameter("int_param3", m_int_param3, "0");
  bindParameter("int_type", m_int_type, "2");

  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t SmoothComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SmoothComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SmoothComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t SmoothComponent::onActivated(RTC::UniqueId ec_id)
{
  osrc = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 3);
  dst = cvCreateImage( cvSize(m_int_width,m_int_height),IPL_DEPTH_8U, 3);
  return RTC::RTC_OK;
}



RTC::ReturnCode_t SmoothComponent::onDeactivated(RTC::UniqueId ec_id)
{
  cvReleaseImage(&osrc);
  cvReleaseImage(&dst);
  return RTC::RTC_OK;
}



RTC::ReturnCode_t SmoothComponent::onExecute(RTC::UniqueId ec_id)
{
  /*inport part*/
  m_InImageDataIn.read();
  if(m_InImageData.data.length()==m_int_width*m_int_height*3){
    memcpy(osrc->imageData,(void *)&(m_InImageData.data[0]),m_int_width*m_int_height*3);

    switch(m_int_type)
     {
    case 0:
      cvSmooth(osrc,dst,CV_BLUR_NO_SCALE,m_int_param1,m_int_param2,m_int_param3);
      break;
     case 1:
       cvSmooth(osrc,dst,CV_BLUR,m_int_param1,m_int_param2,m_int_param3);
       break;
     case 2:
       cvSmooth(osrc,dst,CV_GAUSSIAN,m_int_param1,m_int_param2,m_int_param3);
       break;
     case 3:
       cvSmooth(osrc,dst,CV_MEDIAN,m_int_param1,m_int_param2,m_int_param3);
       break;
     case 4:
       cvSmooth(osrc,dst,CV_BILATERAL,m_int_param1,m_int_param2,m_int_param3);
       break;
     }
    

    /*outport part begin*/
    m_OutImageData.data.length(m_int_width*m_int_height*3);
    memcpy((void *)&(m_OutImageData.data[0]),dst->imageData,m_int_width*m_int_height*3);
    m_OutImageDataOut.write();
    /*outport part end*/
  }

  cvWaitKey(100);
  
  return RTC::RTC_OK;
}
  

/*
RTC::ReturnCode_t SmoothComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SmoothComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SmoothComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SmoothComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t SmoothComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void SmoothComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(smoothcomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<SmoothComponent>,
                             RTC::Delete<SmoothComponent>);
  }
  
};


