// -*- C++ -*-
/*!
 * @file  ThresholdComponent.cpp
 * @brief Threshold Component
 * $Date$
 *
 * $Id$
 */

#include "ThresholdComponent.h"


// Module specification
// <rtc-template block="module_spec">
static const char* thresholdcomponent_spec[] =
  {
    "implementation_id", "ThresholdComponent",
    "type_name",         "ThresholdComponent",
    "description",       "Threshold Component",
    "version",           "0.1",
    "vendor",            "OsakaUniv",
    "category",          "Generic",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.int_width", "320",
    "conf.default.int_height", "240",
    "conf.default.int_paramL", "128",
    "conf.default.int_paramU", "256",
    "conf.default.int_type", "0",
    ""
  };
// </rtc-template>

ThresholdComponent::ThresholdComponent(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_InImageDataIn("InImageData", m_InImageData),
    m_OutImageDataOut("OutImageData", m_OutImageData),
    
    // </rtc-template>
    m_src(0), m_osrc(0), m_dst(0),
    dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("InImageData", m_InImageDataIn);
  
  // Set OutPort buffer
  registerOutPort("OutImageData", m_OutImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

	m_src = NULL;
  m_osrc = NULL;
  m_dst = NULL;

}

ThresholdComponent::~ThresholdComponent()
{
}


RTC::ReturnCode_t ThresholdComponent::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("int_width", m_int_width, "320");
  bindParameter("int_height", m_int_height, "240");
  bindParameter("int_paramL", m_int_paramL, "128");
  bindParameter("int_paramU", m_int_paramU, "256");
  bindParameter("int_type", m_int_type, "0");

  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t ThresholdComponent::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ThresholdComponent::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ThresholdComponent::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ThresholdComponent::onActivated(RTC::UniqueId ec_id)
{

  m_src  = cvCreateImage(cvSize(m_int_width, m_int_height), IPL_DEPTH_8U, 1);
  m_osrc = cvCreateImage(cvGetSize(m_src), IPL_DEPTH_8U, 3);
  m_dst  = cvCreateImage(cvGetSize(m_src), 8, 1 );

  return RTC::RTC_OK;
}



RTC::ReturnCode_t ThresholdComponent::onDeactivated(RTC::UniqueId ec_id)
{
  cvReleaseImage(&m_src);//NULL;
  cvReleaseImage(&m_osrc);//NULL;
  cvReleaseImage(&m_dst);

  return RTC::RTC_OK;
}



RTC::ReturnCode_t ThresholdComponent::onExecute(RTC::UniqueId ec_id)
{

  /*inport part*/
  m_InImageDataIn.read();
  if(m_InImageData.data.length() == m_int_width * m_int_height * 3) {
    memcpy(m_osrc->imageData,
           (void *)&(m_InImageData.data[0]),
           m_int_width * m_int_height * 3);
    
    cvCvtColor(m_osrc, m_src, CV_BGR2GRAY);

    cvCopy(m_src, m_dst, NULL);
    //    IplImage* dst = cvCloneImage(m_src);
    
    switch(m_int_type)
      {
      case 0:
	cvThreshold(m_src, m_dst,
                    m_int_paramL, m_int_paramU, CV_THRESH_BINARY);
	break;
      case 1:
	cvThreshold(m_src, m_dst,
                    m_int_paramL, m_int_paramU, CV_THRESH_BINARY_INV);
	break;
      case 2:
	cvThreshold(m_src, m_dst,
                    m_int_paramL, m_int_paramU, CV_THRESH_TRUNC);
	break;
      case 3:
	cvThreshold(m_src, m_dst,
                    m_int_paramL, m_int_paramU, CV_THRESH_TOZERO);
	break;
      case 4:
	cvThreshold(m_src, m_dst,
                    m_int_paramL, m_int_paramU, CV_THRESH_TOZERO_INV);
	break;
      }
    
    cvCvtColor(m_dst, m_osrc, CV_GRAY2BGR );
    
    /*outport part begin*/
    m_OutImageData.data.length(m_osrc->imageSize);
    memcpy((void *)&(m_OutImageData.data[0]),
           m_osrc->imageData,
           m_osrc->imageSize);
    m_OutImageDataOut.write();
    
    /*outport part end*/
    
  }
  
  cvWaitKey(100);


  return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t ThresholdComponent::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ThresholdComponent::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ThresholdComponent::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ThresholdComponent::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ThresholdComponent::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ThresholdComponentInit(RTC::Manager* manager)
  {
    RTC::Properties profile(thresholdcomponent_spec);
    manager->registerFactory(profile,
                             RTC::Create<ThresholdComponent>,
                             RTC::Delete<ThresholdComponent>);
  }
  
};


